/*
 * sparc/jit.h
 * Common SPARC JIT configuration information.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#ifndef __sparc_jit_h
#define __sparc_jit_h

/**/
/* Native function invocation. */
/**/
#define	CALL_KAFFE_FUNCTION_VARARGS(meth, obj, nargs, argptr)		\
	abort()

/**/
/* Exception handling information. */
/**/

extern struct _thread* currentThread;

/* Structure of exception frame on stack */
typedef struct _exceptionFrame {
        uintp	retbp;
        uintp	retpc;
} exceptionFrame;

/* Is this frame valid (ie. is it on the current stack) ? */
#define	FRAMEOKAY(f)							\
	((f)->retbp >= (int)currentThread->PrivateInfo->stackBase &&	\
	 (f)->retbp < (int)currentThread->PrivateInfo->stackEnd)

/* Get the next frame in the chain */
#define	NEXTFRAME(f)							\
	((exceptionFrame*)(f)->retbp)

/* Extract the PC from the given frame */
#define	PCFRAME(f)							\
	((f)->retpc-1)

/* Get the first exception frame from a subroutine call */
#define	FIRSTFRAME(f, o)						\
	(f) = *((exceptionFrame*)(((uintp)&(o))-8))

/* Call the relevant exception handler (rewinding the stack as
   necessary). */
#define CALL_KAFFE_EXCEPTION(frame, info, obj)				\
	abort()

/**/
/* Register management information. */
/**/

/* Define the register set */
#define	REGISTER_SET							\
	{ /* g0 */	0, 0, Reserved,		0, 0    },		\
	{ /* g1 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* g2 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* g3 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* g4 */	0, 0, Reserved,		0, 0    },		\
	{ /* g5 */	0, 0, Reserved,		0, 0    },		\
	{ /* g6 */	0, 0, Reserved,		0, 0    },		\
	{ /* g7 */	0, 0, Reserved,		0, 0    },		\
	{ /* o0 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o1 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o2 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o3 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o4 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o5 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* o6 */	0, 0, Reserved,		0, 0    },		\
	{ /* o7 */	0, 0, Reserved,		0, 0    },		\
	{ /* l0 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l1 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l2 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l3 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l4 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l5 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l6 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* l7 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i0 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i1 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i2 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i3 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i4 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i5 */	0, 0, Rint|Rref,	0, 0    },		\
	{ /* i6 */	0, 0, Reserved,		0, 0    },		\
	{ /* i7 */	0, 0, Reserved,		0, 0    },		\
	{ /* f0 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f1 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f2 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f3 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f4 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f5 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f6 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f7 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f8 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f9 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f10 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f11 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f12 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f13 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f14 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f15 */	0, 0, Rfloat,		0, 0    },		\
	{ /* f16 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f17 */	0, 0, Reserved,		0, 0    },		\
	{ /* f18 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f19 */	0, 0, Reserved,		0, 0    },		\
	{ /* f20 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f21 */	0, 0, Reserved,		0, 0    },		\
	{ /* f22 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f23 */	0, 0, Reserved,		0, 0    },		\
	{ /* f24 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f25 */	0, 0, Reserved,		0, 0    },		\
	{ /* f26 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f27 */	0, 0, Reserved,		0, 0    },		\
	{ /* f28 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f29 */	0, 0, Reserved,		0, 0    },		\
	{ /* f30 */	0, 0, Rdouble,		0, 0    },		\
	{ /* f31 */	0, 0, Reserved,		0, 0    },		\

/* Number of registers in the register set */
#define	NR_REGISTERS	64

/* Define which registers are used for which return values as seen
 * by the the callee */
#define	RETURN_INT		8	/* o0 */
#define	RETURN_REF		8	/* o0 */
#define	RETURN_LONG_LOW		8	/* o0 */
#define	RETURN_LONG_HIGH	9	/* o1 */
#define	RETURN_FLOAT		32	/* f0 */
#define	RETURN_DOUBLE_LOW	32	/* f0 */
#define	RETURN_DOUBLE_HIGH	33	/* f1 */

/**/
/* Opcode generation. */
/**/

/**/
/* Slot management information. */
/**/

/* Size of each slot */
#define	SLOTSIZE	4

#define	FIRSTARGOFFSET		68
#define	FIRSTLOCALOFFSET	-20
#define	EXTRAFRAMESIZE		112
#define	FRAMEALIGN		8

/* Generate slot offset for an argument */
#define SLOT2ARGOFFSET(_n) \
	(FIRSTARGOFFSET + SLOTSIZE * (_n))

/* Generate slot offset for a local (non-argument) */
#define SLOT2LOCALOFFSET(_n) \
	(FIRSTLOCALOFFSET - SLOTSIZE * (maxTemp + maxLocal + maxStack - (_n)))

#endif
