/*
 * constants.c
 * Constant management.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#define	RDBG(s)

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "gtypes.h"
#include "slots.h"
#include "access.h"
#include "object.h"
#include "constants.h"
#include "object.h"
#include "classMethod.h"
#include "file.h"

/*
 * Read in constant pool from opened file.
 */
constants*
readConstantPool(classFile* fp)
{
	constants* info;
	slots* pool;
	u1* tags;
	int i;
	u1 type;
	u2 len;
	char* uc;
	u2 d2, d2b;
	u4 d4;
	int64 d8;

	info = checked_malloc(sizeof(constants));
	readu2(&info->size, fp);
RDBG(	printf("constant_pool_count=%d\n", info->size);	)

	/* Allocate space for tags and data */
	pool = checked_calloc((sizeof(slots) + sizeof(u1)) * info->size, sizeof(char));
	tags = (u1*)&pool[info->size];
	info->data = pool;
	info->tags = tags;

	pool[0].v.taddr = 0;
	tags[0] = CONSTANT_Unknown;
	for (i = 1; i < info->size; i++) {

		readu1(&type, fp);
RDBG(		printf("Constant type %d\n", type);			)
		tags[i] = type;

		switch (type) {
		case CONSTANT_Utf8:
			readu2(&len, fp);
			pool[i].v.taddr = makeUtf8Const (fp->buf, len);
			fp->buf += len;
			break;
		case CONSTANT_Class:
			readu2(&d2, fp);
			pool[i].v.tint = d2;
			break;

		case CONSTANT_String:
			readu2(&d2, fp);
			pool[i].v.tint = d2;
			break;

		case CONSTANT_Fieldref:
		case CONSTANT_Methodref:
		case CONSTANT_InterfaceMethodref:
		case CONSTANT_NameAndType:
			readu2(&d2, fp);
			readu2(&d2b, fp);
			pool[i].v.tint = (d2b << 16) | d2;
			break;

		case CONSTANT_Integer:
		case CONSTANT_Float:
			readu4(&d4, fp);
			pool[i].v.tint = d4;
			break;

		case CONSTANT_Long:
		case CONSTANT_Double:
			tags[i+1] = CONSTANT_Unknown;
			pool[i+1].v.taddr = 0;
			readu8(&d8, fp);
			pool[i].v.tlong = d8;
			i++;
			break;

		default:
			fprintf(stderr, "Bad constant %d\n", type);
			return (0);
		}
	}

	return (info);
}
